import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

/*
 * New version of SaccadeArt demo, as a Java application,
 * created on Jan. 8, 2015,
 * based on my applet originally created Created on Mar 22, 2006.
 * (C) Copyright, Steve Tanimoto, 2006, 2015.
 */

/**
 * @author Steve Tanimoto
 *
 *         This is an applet whose task is to display a few vertical bars of
 *         rapidly changing colors in such a way that an observer who scans her
 *         eyes across the bars might see a 2D image. The test images are of a
 *         smiley face and a fish.
 */
public class SaccadeArt extends JFrame implements Runnable, ActionListener {
	DispPanel displayPanel;
	BufferedImage smileyImage;
	BufferedImage fishImage;
	BufferedImage testImage;
	JComboBox cb;
	int x0;
	int frameDelay = 5; // millisec.
	int step = 2;
	int imWidth = 256;
	int rightEdge = 1000;
	// int n = imWidth/step;
	int nCols = 4;
	int columnWidth = 1;
	// int columnSep;
	int columnSep;
	int xoffSet;
	int columnSepInImage;
	int frameNum = 0;
	boolean first = true;
	int n = 1;

	void computeParams(int numCols) {
		nCols = numCols;
		columnSep = rightEdge / nCols;
		xoffSet = columnSep / 2;
		columnSepInImage = columnSep / step;
	}

	public SaccadeArt() {
		setLayout(new BorderLayout());
		displayPanel = new DispPanel();

		add(displayPanel, BorderLayout.CENTER);
		// displayPanel.setOpaque(true);
		cb = new JComboBox();
		add(cb, BorderLayout.SOUTH);

		cb.addItem("1");
		cb.addItem("2");
		cb.addItem("3");
		cb.addItem("4");
		cb.addItem("5");
		cb.addItem("6");
		cb.addItem("7");
		cb.addItem("8");
		cb.addItem("9");
		cb.addItem("10");
		cb.addItem("12");
		cb.addItem("16");
		cb.addItem("32");
		cb.setSelectedIndex(3);
		cb.addActionListener(this);
		computeParams(4);
	}

	public void actionPerformed(ActionEvent e) {
		System.out.println("New number of columns selected: ");
		String label = (String) cb.getSelectedItem();
		System.out.println(label);
		nCols = Integer.parseInt(label);
		computeParams(nCols);
	}

	public void paintComponent(Graphics g) {
		displayPanel.paintComponent(g);
	}

	class DispPanel extends JPanel {

		public void paintComponent(Graphics g) {
			// g.drawImage(smileyImage, 0, 0, imWidth, rightEdge, this);
			// if (first) {
			g.setColor(Color.BLACK);
			g.fillRect(0, 0, rightEdge, 800);
			first = false;
			// g.setColor(Color.BLACK);
			// } else {
			x0 = (frameNum * step) % imWidth;

			for (int c = 0; c < nCols; c++) {
				int xc = rightEdge - (c * columnSep) - xoffSet;
				int xcInImage = c * columnSepInImage;
				x0 = (xcInImage + (frameNum * step)) % imWidth;
				g.drawImage(testImage, xc, 0, xc + columnWidth, 512, x0, 0,
						x0 + 3, 256, this);
			}
			// }
		}

	}

	public void run() {
		try {
			Thread.currentThread().sleep(1000);
		} catch (Exception e) {
		}
		for (frameNum = 0; true; frameNum++) {
			repaint();
			try {
				Thread.currentThread().sleep(frameDelay);
				// System.out.print(".");
			} catch (Exception e) {
			}
			if ((frameNum * frameDelay / 1024) % 16 > 7) {
				testImage = fishImage;
			} else {
				testImage = smileyImage;
			}
		}
	}

	public static void main(String[] argv) {
		SaccadeArt f;
		// Print a copyright message.
		System.out.println("A demonstration of saccade art.");
		System.out.println("This program is copyrighted.");
		System.out.println("(C) Steven Tanimoto, 2006, 2015");
		System.out.println("All rights reserved.");
		// Set up a separate frame with a black background.
		// It's sized so that it's likely to fill a typical screen.
		f = new SaccadeArt();
		f.setSize(1000, 800);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setTitle("Saccade Art: How low can you go and see the show?");

		f.show();
		// Read in the test images.
		try {
			System.out.println("Trying to load the test images...");
			f.fishImage = ImageIO.read(new File("SaccadeFish.gif"));
			f.smileyImage = ImageIO.read(new File("SmileyFace.gif"));
		} catch (Exception e) {
			System.out
					.println("There was a problem loading one or more of the images.");
		}
		// System.out.println("The images seems to have been loaded.");
		Thread displayThread = new Thread(f);
		displayThread.start();
	}
}
